#!/usr/bin/env bash


function usage {
    log "usage:

    vue-init-watcher.sh <template_project_dir> <output_project_name>


overview
========

	watch source files of a vue.js custom template project, and triggers vue-cli to rebuild the project.
	to be used while developing custom templates.


arguments
=========

	template_project_dir

		the local path of the vue-cli custom template project to watch.

	output_project_name

		an name for the project that will be generated by vue-cli.

 "
}

function main {
	local source_dir="$( cd "$(dirname "${BASH_SOURCE}")" ; pwd -P )"
    validate_args 3 "$@"
    validate_os
    start "$@"
}

function start {
	local template_project_dir="$1"
	local output_dir="$2"
	local output_project_name="$3"
	local output_project_dir="$output_dir/$output_project_name"
	local expect_file="vue-init.exp"
	local expect_path="${source_dir}/${expect_file}"

#    trap  'rm -f '"${expect_path}" EXIT

	ensure_fswatch
	ensure_expect

	# create a directory to force vue-init to ask about overwriting the directory on the first time
	if [[ ! -d "$output_project_dir" ]]; then
    	mkdir "$output_project_dir"
	fi

	autoexpect -f ${expect_path} vue init "${template_project_dir}" "${output_project_name}"

	line_replace ${expect_path} 'set timeout -1' 'set timeout 5'
	line_replace ${expect_path} 'expect eof' 'interact'

	chmod u+x ${expect_path}

	log "watching template files in [${template_project_dir}/template]..."
	fswatch -o "${template_project_dir}/template" | while read num; do
		log "template source has changed, triggering vue-cli with default choices..."
		${expect_path} ${template_project_dir} ${output_project_name}
	done
}

function line_replace {
	local file="$1"
	local from="$2"
	local to="$3"
	# bsd/macos specific implementation
	sed -e 's/^'"$from"'$/'"$to"'/' "$file" >"${file}.new"
	mv -- "${file}.new" "$file"
	rm -f "${file}.new"
}

function ensure_fswatch {
	if ! hash fswatch 2>/dev/null; then
		log "fswatch is not installed. installing via brew..."
		# bsd/macos specific implementation
        brew install fswatch
	fi
}

function ensure_expect {
	if ! hash expect 2>/dev/null || ! hash autoexpect 2>/dev/null; then
		log "expect or autoexpect are not installed. installing via brew..."
		# bsd/macos specific implementation
        brew install expect
	fi
}

function validate_args {
    local min=$1; shift
    if (($# < $min)); then
        usage
        exit 1
    fi
}

function validate_os {
	if [[ $OSTYPE = "darwin"* ]]; then # mac
		return 0
	elif [[ $OSTYPE = "linux-gnu" ]]; then # linux
		log "linux is not supported, sorry..."
		exit 1
	elif [[ $OSTYPE = "msys" ]]; then # windows (mingw/git-bash)
		log "windows is not supported, sorry..."
		exit 1
	fi
}

function log {
	local msg="$1"
	printf "\n[vue-init-watcher] %s\n" "$msg"
}


main "$@"
